/*

					Logic Scope AVR

				   Support Functions

			Copyrights 2009,  Yoshi Hiroshi

*/

#include <avr/io.h>
#include "usitwi2313.h"
#include "uart.h"
#include <util/setbaud.h>

//**********************************************************
//		Init UART
//**********************************************************
//
//----------------------------------------------------------
void initUART()
{
	UART_BAUD_RATE_REGISTER_H = UBRRH_VALUE;
	UART_BAUD_RATE_REGISTER_L = UBRRL_VALUE;
#if USE_2X
	UART_TRL_STATUS_A |= _BV(UART_2X_SPEED_BIT);
#else
	UART_TRL_STATUS_A &= ~_BV(UART_2X_SPEED_BIT);
#endif
	UART_TRL_STATUS_C = UART_CHARACTER_SIZE;
	UART_TRL_STATUS_B = _BV(UART_TRANSMITTER_ENABLE_BIT) | _BV(UART_RECIEVER_ENABLE_BIT);
	UART_TRANSMITTER_PORT_DIRECTION_SET;
}

//**********************************************************
//		Recieve Data from UART
//**********************************************************
//
//----------------------------------------------------------
unsigned char readUART(unsigned char flag)
{
	if(!flag)
	{
		if((UART_TRL_STATUS_A & _BV(UART_RECIEVE_COMPLETE_BIT)) == 0)
			return 0;
	}
	unsigned char val;
	while((UART_TRL_STATUS_A & _BV(UART_RECIEVE_COMPLETE_BIT)) == 0);
	val = UART_DATA_REGISTER;
//	UDR1 = val;
	return val;
}

unsigned char checkReceiveUART()
{
	return ((UART_TRL_STATUS_A & _BV(UART_RECIEVE_COMPLETE_BIT)) != 0);
}

//**********************************************************
//		Send Data via UART
//**********************************************************
//
//----------------------------------------------------------
void sendUART(unsigned char data)
{
	while((UART_TRL_STATUS_A & _BV(UART_DATA_EMPTY_BIT)) == 0);
	UART_DATA_REGISTER = data;
}

void inhibitReceiveUART(unsigned char flag)
{
}

//**********************************************************
//		Send Data via UART (Hex Format)
//**********************************************************
void sendUART_HEX(unsigned char data)
{
	unsigned char halfLeft = data >> 4;
	unsigned char halfRight = data & 0x0F;
	halfLeft = (halfLeft >= 10) ? (halfLeft + 'A' - 10) : (halfLeft + '0');
	halfRight = (halfRight >= 10) ? (halfRight + 'A' - 10) : (halfRight + '0');
	sendUART(halfLeft);
	sendUART(halfRight);
}

//**********************************************************
//		Send Terminator via UART
//**********************************************************
//
//----------------------------------------------------------
void sendTermUART()
{
	sendUART(0x0D);
	sendUART(0x0A);
}

//**********************************************************
//		Check Terminator
//**********************************************************
//
//----------------------------------------------------------
void checkTermUART()
{
	volatile unsigned long waitLimit = 1000000;
	while((readUART(UART_READ_RETURN_IMMIDIATE) != 0x0A) && (--waitLimit > 0));
}

//**********************************************************
//		Recieve Data from UART (Hex Format)
//**********************************************************
//----------------------------------------------------------
unsigned char readUART_HEX()
{
	unsigned char data[2];
	unsigned char val = 0;
	data[0] = readUART(UART_READ_WAIT_COMPLETION);
	if(((data[0] >= '0') && (data[0] <= '9')) ||
		((data[0] >= 'A') && (data[0] <= 'F'))||
		((data[0] >= 'a') && (data[0] <= 'f')))
	{
		data[1] = readUART(UART_READ_WAIT_COMPLETION);
		if(((data[1] >= '0') && (data[1] <= '9')) ||
			((data[1] >= 'A') && (data[1] <= 'F'))||
			((data[1] >= 'a') && (data[1] <= 'f')))
		{
			val = ((data[0] > '9') ? ((data[0] & 0x07) + 9) : (data[0] - '0')) << 4;
			val |= (data[1] > '9') ? ((data[1] & 0x07) + 9) : (data[1] - '0');
		}
	}
	return val;
}


