/*

					Logic Scope AVR

				   Support Functions

			Copyrights 2009,  Yoshi Hiroshi

*/


#include "usitwi2313.h"

//------------------------------
void sendStartCondition()
{
	TWI_SCL_FREE;			// free SCL line
	while( TWI_SCL_CURR_VAL == 0 );
	_delay_us(DELAY_LONG);
	TWI_SDA_SET_LOW;		// Initiate SDA line
	_delay_us(DELAY_LONG);
	TWI_SCL_SET_LOW;		// Initiate SCL line
//	USISR = _BV(USISIF);	// Clear Start Detected Flag
}

//------------------------------
unsigned char transferTWIdata(unsigned char val, unsigned char bitLength)
{
	unsigned char result;
	TWI_DATA_REGISTER = val;
	TWI_SDA_FREE;			// free SDA line
	USISR = _BV(USISIF) | _BV(USIOIF) | _BV(USIPF) | _BV(USIDC) |
			((16 - (bitLength << 1)) & 0x0F);
	do {
		_delay_us(DELAY_LONG);
		USICR |= _BV(USICLK) | _BV(USITC);	// toggle SCL to high(free)
		while( TWI_SCL_CURR_VAL == 0 );
		_delay_us(DELAY_LONG);
		USICR |= _BV(USICLK) | _BV(USITC);	// toggle SCL to low
	}while(!(USISR & _BV(USIOIF)) );
	_delay_us(DELAY_LONG);
	result = TWI_DATA_REGISTER;
	TWI_SDA_FREE_ONLY_DR;
	TWI_SDA_AS_OUTPUT;
	return result;
}

//------------------------------
unsigned char getAckData()
{
	TWI_SDA_AS_INPUT;
	return (transferTWIdata(0, 1) & 0x01);
}

//------------------------------
void sendStopCondition()
{
	TWI_SDA_SET_LOW;
	_delay_us(DELAY_SHORT);
	TWI_SCL_FREE;
	while( TWI_SCL_CURR_VAL == 0 );
	_delay_us(DELAY_LONG);
	TWI_SDA_FREE;
	_delay_us(DELAY_LONG);
}

//------------------------------
void initTWI()
{
	TWI_SCL_FREE;
	TWI_SDA_FREE_WITH_DR;
	TWI_SCL_AS_OUTPUT;
	TWI_SDA_AS_OUTPUT;
	USICR = _BV(USIWM1) 
//			| _BV(USIWM0) 
			| _BV(USICS1) 
//			| _BV(USICS0) 
			| _BV(USICLK) 
//			| _BV(USITC) 
			;
	USISR = _BV(USISIF) | _BV(USIOIF) | _BV(USIPF) | _BV(USIDC);
}

//------------------------------
unsigned char doTWIrequest(struct TWIreq* req)
{
	unsigned char transferCount = req->commandLength + req->recieveLength;
	if(transferCount == 0) return 0;
	if(req->commandLength != 0)
	{
// o Send start
		sendStartCondition();
// o Send Slave address & read/write flga
		transferTWIdata(((req->slaveAdrs << 1) | TWI_REQUEST_WRITE), 8);
// o Check ACK
		if(!getAckData())
		{
			transferCount = 0;
			while(transferCount < req->commandLength)
			{
// o Send Command
				transferTWIdata(req->commandBuffer[transferCount++], 8);
// o Check ACK
				if(getAckData() != TWI_RESPONCE_ACK)
				{
				//	return 0;
				}
			}
		}
	}
// if read request
	if(req->recieveLength != 0)
	{
		transferCount = 0;
// o Send start
		sendStartCondition();
// o Send Slave address & read/write flga
		transferTWIdata(((req->slaveAdrs << 1) | TWI_REQUEST_READ), 8);
// o Check ACK
		if(!getAckData())
		{
//   o Read data
			while(transferCount < req->recieveLength)
			{
//       read 8bit data
				TWI_SDA_AS_INPUT;
				req->recieveBuffer[transferCount] =
						transferTWIdata(req->recieveBuffer[transferCount], 8);
				transferCount++;
//       send ACK
				if(transferCount < req->recieveLength)
				{
					transferTWIdata((TWI_RESPONCE_ACK << 7), 1);
				} else {
					transferTWIdata((TWI_RESPONCE_NACK << 7), 1);
				}
			}
		}
		req->actualLength = transferCount;
	}
// o Send Stop
	sendStopCondition();
//
	return 0;
}

